function WS4Redis(options, $) {
    'use strict';
    var opts, ws, deferred, timer, attempts = 1, must_reconnect = true;
    var heartbeat_interval = null, missed_heartbeats = 0;

    if (this === undefined)
        return new WS4Redis(options, $);
    if (options.uri === undefined)
        throw new Error('No Websocket URI in options');
    if ($ === undefined)
        $ = jQuery;
    opts = $.extend({ heartbeat_msg: null }, options);
    var uri = opts.uri;

    this.connect = function () {
        try {
            if (ws && (is_connecting() || is_connected())) {
                //console.log("Websocket is connecting or already connected.");
                return;
            }
            
            if ($.type(opts.connecting) === 'function') {
                opts.connecting();
            }
            
            deferred = $.Deferred();
            ws = new WebSocket(uri);
            ws.onopen = on_open;
            ws.onmessage = on_message;
            ws.onerror = on_error;
            ws.onclose = on_close;
            timer = null;
            //console.log('connected');
        } catch (err) {
            //console.log(err);
            try_to_reconnect();
            deferred.reject(new Error(err));
        }
    }

    function try_to_reconnect() {
        if (must_reconnect && !timer) {
            // try to reconnect
            //console.log('Reconnecting...');
            var interval = generate_inteval(attempts);
            timer = setTimeout(function() {
                attempts++;
                connect(ws.url);
            }, interval);
        }
    }
    
    function send_heartbeat() {
        try {
            missed_heartbeats++;
            if (missed_heartbeats > 3)
                throw new Error("Too many missed heartbeats.");
            ws.send(opts.heartbeat_msg);
        } catch(e) {
            clearInterval(heartbeat_interval);
            heartbeat_interval = null;
            //console.warn("Closing connection. Reason: " + e.message);
            if ( !is_closing() && !is_closed() ) {
                ws.close();
            }
        }
    }

    function on_open() {
        attempts = 1;
        deferred.resolve();
        if (opts.heartbeat_msg && heartbeat_interval === null) {
            missed_heartbeats = 0;
            heartbeat_interval = setInterval(send_heartbeat, 5000);
        }
        if ($.type(opts.connected) === 'function') {
            opts.connected();
        }
    }

    function on_close(evt) {
        //console.log("Connection closed!");
        if ($.type(opts.disconnected) === 'function') {
            opts.disconnected(evt);
        }
        try_to_reconnect();
    }

    function on_error(evt) {
        //console.error("Websocket connection is broken!");
        deferred.reject(new Error(evt));
    }

    function on_message(evt) {
        if (opts.heartbeat_msg && evt.data === opts.heartbeat_msg) {
            missed_heartbeats = 0;
        } else if ($.type(opts.receive_message) === 'function') {
            return opts.receive_message(evt.data);
        }
    }

    function generate_inteval(k) {
        var maxInterval = (Math.pow(2, k) - 1) * 1000;
        if (maxInterval > 30*1000) {
            maxInterval = 30*1000;
        }
        return Math.random() * maxInterval;
    }

    this.send_message = function(message) {
        ws.send(message);
    };
    
    this.get_state = function() {
        return ws.readyState;   
    };
    
    function is_connecting() {
        return ws && ws.readyState === 0;   
    }
    
    function is_connected() {
        return ws && ws.readyState === 1;   
    }
    
    function is_closing() {
        return ws && ws.readyState === 2;   
    }

    function is_closed() {
        return ws && ws.readyState === 3;   
    }
    
    
    this.close = function () {
        clearInterval(heartbeat_interval);
        must_reconnect = false;
        if (!is_closing() || !is_closed()) {
            ws.close();
        }
    }
    
    this.is_connecting = is_connecting; 
    this.is_connected = is_connected;
    this.is_closing = is_closing;
    this.is_closed = is_closed;
};

jQuery(document).ready(function($) {

    var ws4redis = WS4Redis({
        uri: 'ws://push.zournal.org/ws/notifiche?subscribe-broadcast&publish-broadcast&echo',
        receive_message: receiveMessage,
        heartbeat_msg: null
    });

    if (("Notification" in window)) {
        var permission = Notification.permission;
    }

    function receiveMessage(msg) {

      if (("Notification" in window)) {
        if ($(this).getNotificationPermissions() === 'true') {
            var notification = new Notification('La Stampa', {body: msg.split('|||')[0], icon: 'https://s3-eu-west-1.amazonaws.com/8080-distrib/lastampait/ls_icon.png', requireInteraction: false});
            notification.onclick = function(event) {
                event.preventDefault();
                window.open(msg.split('|||')[1], '_blank');
                notification.close()
            }
            setTimeout(function() {
                notification.close()
            }, 20000);
        }
      }
    };
    

    /*   Add V2 */
    $.fn.toggleNotificationPermissions = function(input) {

      if (("Notification" in window)) {
        if (Notification.permission === 'granted') {
            localStorage.setItem('notification-permissions', input.checked ? 'true' : 'false');

            if (input.checked === true) {
                ws4redis.connect();
            }
        }
        else if (Notification.permission === 'denied') {
            localStorage.setItem('notification-permissions', 'false');
            input.checked = false;
        }
        else if (Notification.permission === 'default') {
            Notification.requestPermission(function(choice) {
                if (choice === 'granted') {
                    localStorage.setItem('notification-permissions', input.checked ? 'true' : 'false');
                    ws4redis.connect();
                } else {
                    localStorage.setItem('notification-permissions', 'false');
                    input.checked = false;
                }
            });
        }
      }
    };

    /*   Add V2 */
    $.fn.getNotificationPermissions = function() {
      if (("Notification" in window)) {
        if (Notification.permission === 'granted') {
            if (typeof(Storage) !== "undefined") {
                if ( ! localStorage.getItem('notification-permissions')) {
                    return 'true';
                }
                else {
                    return localStorage.getItem('notification-permissions');
                }
            }
            else {
                return 'true'
            }

        } else {
            return Notification.permission;
        }
      }
    };

    if ($(this).getNotificationPermissions() === 'true') {
        ws4redis.connect();
    }

});
